@@ -29,6 +29,11 @@ def run_routine(args):
     # Load routine configs
     try:
         configs_routine = load_config(args.config)
+        # Normalize the routine configs properties
+        try:
+            _ = configs_routine['constraints']
+        except KeyError:
+            configs_routine['constraints'] = None
     except Exception as e:
         logging.error(e)
         return
@@ -56,4 +61,5 @@ def run_routine(args):
     try:
         run(routine, args.yes, args.save, args.verbose)
     except Exception as e:
+        raise e
         logging.error(e)
